const state = {
    cartItems: [],
    cartTotal: 0,
    itemPriced: null
}

const getters = {
    cartItems(state) {
        return state.cartItems
    },
    cartTotal(state) {
        return state.cartTotal
    },
    itemPriced(state) {
        return state.itemPriced
    }
}

const mutations = {
    setCartItems(state, payload) {
        state.cartItems= [...payload]
    },
    pushToCart(state, payload) {
        let productFound = state.cartItems.find(item => item.id === payload.id)

        if(productFound) {
            if (payload.quantity > 1) {
                if (productFound.quantity === payload.quantity) {
                    productFound.quantity = productFound.quantity + 1    
                } else {
                    productFound.quantity = payload.quantity
                }
            } else {
                productFound.quantity = productFound.quantity + 1
            }
            return
        }

        state.cartItems = [payload, ...state.cartItems]
    },
    pullFromCart(state, payload) {
        let foundIndex = state.cartItems.findIndex(item => item.id === payload.item.id)

        if(state.cartItems[foundIndex].quantity === 1) {
            payload.mode = 'full'
        }

        if (payload.mode === 'partial') {
            if (payload.item.quantity !== -1) {
                state.cartItems[foundIndex].quantity = payload.item.quantity
            } else {
                state.cartItems[foundIndex].quantity = state.cartItems[foundIndex].quantity - 1
            }
        } else {
            state.cartItems.splice(foundIndex, 1)
        }
    },
    setCartTotal(state, payload) {
        state.cartTotal = payload
    },
    setItemPriced(state, payload) {
        state.itemPriced = payload
    },
    applyPrice(state, payload) {
        let foundProduct = state.cartItems.find(item => item.id === state.itemPriced.id)
        foundProduct.price = payload
    },
    undoPrice(state, payload) {
        let foundProduct = state.cartItems.find(item => item.id === payload.id)
        foundProduct.price = foundProduct.standardPrice
    }
}

const actions = {
    addToCart({ commit }, payload) {
        commit('pushToCart', payload)
    },
    removeFromCart({ commit }, payload) {
        commit('pullFromCart', {
            item: payload,
            mode: 'full'
        })
    },
    changeCartTotal ({ commit, dispatch }, payload) {
        commit('setCartTotal', payload)
        dispatch('changeAmountToPay', payload)
    },
    decreaseFromCart({ commit }, payload) {
        commit('pullFromCart', {
            item: payload,
            mode: 'partial'
        })
    },
    changePrice({ commit }, payload) {
        if (typeof payload === 'number') {
           commit('applyPrice', payload)
        }

        if (typeof payload === 'object') {
            commit('setItemPriced', payload)
            return
        }

        commit('setItemPriced', null)

    },
    undoPrice({ commit }, payload) {
        commit('undoPrice', payload)
    },
    updateCartTotal({ commit, dispatch }, payload) {
        commit('setCartTotal', payload)
        dispatch('changeAmountToPay', payload)
    },
    updateFromPurchaseOrder({ commit, dispatch }, purchaseOrder) {
        dispatch('resetCart')

        if (!purchaseOrder) {
            return
        }

        let total = 0
        for (let line of purchaseOrder.lines) {
            total = total + (line.price * line.quantity)

            commit('pushToCart', line)
        }

        dispatch('changeCartTotal', total)
    },
    resetCart({ commit }) {
        commit('setCartItems', [])
        commit('setCartTotal', 0)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}