const state = {
    journals: [],
    loadingJournals: true,
    selectedJournal: null
}

const getters = {
    journals(state) {
        return state.journals
    },
    bankJournals(state, getters) {
        return state.journals.filter(j => j.type === 'bank' && j.currencyId === getters.selectedCurrency.id)
    },
    selectedJournal(state) {
        return state.selectedJournal
    },
    setLoadingJournals(state, payload) {
        return state.loadingJournals
    },
    hasJournalSelected(state) {
        return !!state.selectedJournal
    },
    hasCashJournalSelected(state) {
        return !!state.selectedJournal && state.selectedJournal.type === 'cash'
    },
    hasBankJournalSelected(state) {
        return !!state.selectedJournal && state.selectedJournal.type === 'bank'
    }
}

const mutations = {
    setJournals(state, payload) {
        state.journals = [...payload]
    },
    autoSelectJournal(state) {
        state.selectedJournal = state.journals.find(j => j.type === 'cash')
    },
    setSelectedJournal(state, payload) {
        if (!payload) {
            state.selectedJournal = payload
            return
        }

        state.selectedJournal = state.journals.find(item => item.id === payload)
    },
    setLoadingJournals(state, payload) {
        state.loadingJournals = !!payload
    }
}

const actions = {
    initJournals({ commit }, payload) {
        commit('setJournals', payload)
        commit('autoSelectJournal')
        commit('setLoadingJournals', false)
    },
    autoSelectJournal({ commit }) {
        commit('autoSelectJournal')
    },
    selectJournal({ commit }, payload) {
        commit('setSelectedJournal', payload)
    },
    resetJournal({ commit }, payload) {
        commit('setLoadingJournals', false)
        commit('setJournals', [])
        commit('setSelectedJournal', null)
        commit('autoSelectJournal')
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}