const state = {
    purchaseOrders: [],
    filteredPurchaseOrders: [],
    loadingPurchaseOrders: false,
    selectedPurchaseOrder: null
}

const getters = {
    loadingPurchaseOrders(state) {
        return state.loadingPurchaseOrders
    },
    purchaseOrders(state) {
        return state.purchaseOrders
    },
    visiblePurchaseOrders(state) {
        return state.filteredPurchaseOrders.length === 0 ? state.purchaseOrders : state.filteredPurchaseOrders
    },
    selectedPurchaseOrder(state) {
        return state.selectedPurchaseOrder
    }
}

const mutations = {
    setLoadingPurchaseOrder(state, loading) {
        state.loadingPurchaseOrders = !!loading
    },
    setPurchaseOrders(state, purchaseOrders) {
        state.purchaseOrders = purchaseOrders
    },
    setFilteredPurchaseOrders(state, filteredPurchaseOrders) {
        state.filteredPurchaseOrders = filteredPurchaseOrders
    },
    setSelectedPurchaseOrder(state, purchaseOrder) {
        state.selectedPurchaseOrder = purchaseOrder
    }
}

const actions = {
    initPurchaseOrders({ commit }, purchaseOrders) {
        commit('setPurchaseOrders', purchaseOrders)
        commit('setLoadingPurchaseOrder')
    },
    filterPurchaseOrders({ commit }, filtered) {
        commit('setFilteredPurchaseOrders', filtered)
    },
    selectPurchaseOrder({ commit, dispatch }, purchaseOrder) {
        commit('setSelectedPurchaseOrder', purchaseOrder)
        dispatch('updateFromPurchaseOrder', purchaseOrder)
    },
    resetPurchaseOrder({ commit }) {
        commit('setLoadingPurchaseOrder', true)
        commit('setPurchaseOrders', [])
        commit('setSelectedPurchaseOrder', null)
    }
}

export default {
    state,
    getters,
    actions,
    mutations
}